# -*- coding: utf-8 -*-
from peltak.actions import actions
from peltak.common.utils import fetch_template
from peltak.config import MR_TEMPLATE_PATH
from peltak.core import cstr


@actions.register(
    'fetch-template',
    desc=("Fetch template from gitlab issue. The issue title and label can "
          "be defined in the config using 'issue_title' and 'issue_label'.")
)
def action_fetchtemplate(app):
    app.authenticate()

    template = fetch_template(app)
    if template is None:
        print(cstr('\n'.join([
            "^31No issue with title ^1{title}^0 ^31and label ^1{label}^0",
            "",
            "You can customize the title and label to search for with",
            "the following ",
            "^1mr_template_title^0",
            "^1issue_label^0."
        ]).format(
            title = app.conf.mr_template_title,
            label = app.conf.issue_label
        )))
    else:
        with open(MR_TEMPLATE_PATH, 'w') as fp:
            fp.write(template)
        print(template)





#!/usr/bin/env python2
# -*- coding: utf-8 -*-
from __future__ import absolute_import
import requests
import copy
from igor.jsutil import jsobj, json_loads, json_dumps
from six import PY2, PY3
if PY2:
    from urlparse import urljoin
    from urllib import quote as urlescape
elif PY3:
    from urllib.parse import urljoin, quote as urlescape


class ApiError(RuntimeError):
    """ API error exception class.

    There is an aditional ``response`` field that contains the erranous
    response.
    """
    def __init__(self, response):
        super(ApiError, self).__init__("{}:\n{}".format(
            response.status_code,
            response.content
        ))
        self.response = response


class ApiClient(object):
    """ Wrapper for a HTTP endpoint.

    This will allow HTTP calls using *requests* interface. It also allows
    to store globally a set of defaults for each call. All values will be
    passed as arguments to *requests* and headers will be merged with those
    in the request.
    """
    def __init__(self, endpoint):
        self.endpoint = endpoint
        self.defaults = jsobj()

    def _process_args(self, args):
        ret = copy.copy(self.defaults)
        if 'headers' in self.defaults and 'headers' in args:
            headers = self.defaults['headers']
            headers.update(args['headers'])
            args['headers'] = headers
        ret.update(args)
        return ret

    def get(self, url, **kwargs):
        args = self._process_args(kwargs)
        u = urljoin(self.endpoint, url)
        return requests.get(urljoin(self.endpoint, url), **args)

    def post(self, url, **kwargs):
        args = self._process_args(kwargs)
        return requests.post(urljoin(self.endpoint, url), **args)

    def put(self, url, **kwargs):
        args = self._process_args(kwargs)
        return requests.put(urljoin(self.endpoint, url), **args)

    def delete(self, url, **kwargs):
        args = self._process_args(kwargs)
        return requests.delete(urljoin(self.endpoint, url), **args)


class Gitlab(object):
    """ Gitlab API """

    def __init__(self, url, token=None, verify_ssl=True):
        self.api = ApiClient(urljoin(url, '/api/v3/'))
        self._token = None
        self.token = token
        self.users = None
        self.branches = None
        self.api.defaults['verify'] = verify_ssl

    def _parse_response(self, r):
        try:
            content = json_loads(r.content)
            if isinstance(content, list):
                return [jsobj(item) for item in content]
            return jsobj(r.content)
        except:
            return jsobj()

    @property
    def logged_in(self):
        """ Return ``True`` if the instance is logged in (has token) """
        return self.token is not None

    @property
    def token(self):
        return self._token

    @token.setter
    def token(self, value):
        """ Set token.

        The token will be passed in each request for authorization.
        """
        self._token = value
        if value is not None:
            headers = self.api.defaults.setdefault('headers', {})
            headers['PRIVATE-TOKEN'] = value

    def login(self, username, password):
        """ Login to remote Gitlab instance. """
        r = self.api.post('session', data={
            'login': username,
            'password': password,
        })
        if r.status_code != 201:
            raise ApiError(r)
        user = self._parse_response(r)
        self.token = user.get('private_token')
        return user

    def current_user(self):
        """ Return currently logged in users """
        r = self.api.get('user')
        if r.status_code != 200:
            raise ApiError(r)
        return self._parse_response(r)

    def list_users(self, force_fetch=False):
        """ List all users on the remote instance.

        :arg force_fetch:   If set to ``True``. This will force HTTP request
                            to fetch the user list from remote instance.
                            Otherwise it will use cached version.
        """
        if force_fetch or self.users is None:
            r = self.api.get('users', params={'per_page': 100})

            if r.status_code != 200:
                raise ApiError(r)

            self.users = [jsobj(u) for u in r.json()]

        return self.users

    def list_branches(self, project_id, force_fetch=False):
        """ List remote branches.

        :arg project_id:    ID of the project for which the branches will be
                            listed.
        :arg force_fetch:   If set to ``True``. This will force HTTP request
                            to fetch the branch list from remote instance.
                            Otherwise it will use cached version.
        """
        if force_fetch or self.branches is None:
            url = 'projects/{}/repository/branches'.format(project_id)
            r = self.api.get(url, params={'per_page': 100})

            if r.status_code != 200:
                raise ApiError(r)

            self.branches = [jsobj(u) for u in r.json()]

        return self.branches

    def has_user(self, username):
        """ Check if the user with the given *username* exists. """
        users = self.list_users()
        return username in (u.username for u in users)

    def find_user(self, username):
        """ Return the user with the given *username* if it exists. """
        return next((u for u in self.list_users() if u.username == username),
                    None)

    def get_project(self, id):
        """ Return the project with the given ID.

        :arg id:    Either project ID as int or a NAMESPACE/PROJECT string.
        """
        url = 'projects/{}'.format(urlescape(str(id), safe=''))
        r = self.api.get(url)
        if r.status_code != 200:
            raise ApiError(r)
        return self._parse_response(r)

    def get_mr(self, project_id, mr_id):
        """ Get MR by ID.

        :arg project_id:    Project ID for which we're requesting a MR.
        :arg mr_id:         The merge request ID.
        """
        url = 'projects/{}/merge_requests?iid={}'.format(project_id, mr_id)
        r = self.api.get(url)
        if r.status_code != 200:
            raise ApiError(r)
        results = self._parse_response(r)
        if results:
            return results[0]
        return jsobj()

    def create_mr(self, project_id, mr):
        """ Create merge request.

        :arg project_id:    Project ID for which we're creating a MR.
        :arg mr:            A dictionary like object that contains the MR
                            details. For more information consult Gitlab API
                            reference.
        """
        project_id = urlescape(str(project_id), safe='')
        r = self.api.post(
            'projects/{}/merge_requests'.format(project_id),
            headers = {'content-type': 'application/json'},
            data    = json_dumps(mr)
        )
        if r.status_code != 201:
            raise ApiError(r)
        return self._parse_response(r)

    def update_mr(self, project_id, mr_id, values):
        """ Update merge request.

        :arg project_id:    Project ID for which we're creating a MR.
        :arg mr_id:         The ID of the merge request to change.
        :arg mr:            A dictionary like object that contains the values
                            to update in the merge request.
        """
        url = 'projects/{}/merge_request/{}'.format(project_id, mr_id)
        r = self.api.put(
            url,
            headers = {'content-type': 'application/json'},
            data    = json_dumps(values),
        )
        if r.status_code != 200:
            raise ApiError(r)
        return self._parse_response(r)




# -*- coding: utf-8 -*-
from __future__ import absolute_import
import os
import os.path
from collections import OrderedDict
from six import string_types
from igor.jsutil import json_dumps, json_loads
from ..core import ensure_dirs


class CachedObj(OrderedDict):
    def __init__(self, *args, **kw):
        self._cache_path = None
        if len(args) == 1 and isinstance(args[0], string_types):
            for key, value in json_loads(args[0], CachedObj).items():
                self[key] = value
        else:
            super(CachedObj, self).__init__(*args, **kw)

    @classmethod
    def load_file(self, path):
        ensure_dirs(path)
        ret = None
        if os.path.exists(path):
            with open(path) as fp:
                json = fp.read() or '{}'
                ret = json_loads(json, CachedObj)
        else:
            ret = CachedObj()

        ret._set_cache_path(path)
        return ret

    def reset(self):
        for key in self:
            del self[key]
        if os.path.exists(self._cache_path):
            os.remove(self._cache_path)

    def _set_cache_path(self, path):
        self._cache_path = path
        if len(self) > 0:
            self._save_changes()

    def _save_changes(self):
        if self._cache_path is not None:
            with open(self._cache_path, 'w') as fp:
                fp.write(json_dumps(self))

    def members(self):
        return dict(self.items())

    def __repr__(self):
        return '<{0}>'.format(str(dict(self)))

    def __str__(self, indent = None):
        """ Use JSON as string representation """
        return str(self.__unicode__(indent))

    def __unicode__(self, indent = None):
        return json_dumps(self, indent=indent)

    def __setattr__(self, name, value):
        try:
            if name in ('_cache_path',) or name.startswith('_'):
                super(CachedObj, self).__setattr__(name, value)
            else:
                self[name] = value
                self._save_changes()
        except:
            import ipdb; ipdb.set_trace()

    def __getattr__(self, name):
        try:
            return self[name]
        except KeyError:
            raise AttributeError("No attribute {}".format(name))




# -*- coding: utf-8 -*-
from peltak.core import git
from peltak.actions import actions


@actions.register(
    'review',
    desc=("Review changes in the current branch against the master. This will "
          "use 'git difftool' to do the comparison. The actual tool used has "
          "to be configured in git.")
)
def action_review(app):
    git.review_branch()




# -*- coding: utf-8 -*-
from __future__ import absolute_import
import readline
import os.path
from six import PY2, PY3
from ..config import MR_TEMPLATE_PATH
from ..core import git, cstr
from ..core.template import Template
if PY2:
    from urlparse import urljoin
elif PY3:
    from urllib.parse import urljoin


class UserCompleter(object):
    def __init__(self, users):
        self.users = users

    def complete(self, text, state):
        response = None
        if state == 0:
            # This is the first time for this text, so build a match list.
            if text:
                self.matches = [u.username for u in self.users if all((
                    u.username.startswith(text),
                    u.state == 'active'
                ))]
            else:
                self.matches = self.options[:]

        # Return the state'th item from the match list,
        # if we have that many.
        try:
            response = self.matches[state]
        except IndexError:
            response = None
        return response


def yesno(question, default=None):
    prompt = cstr('^1{question} {options} ^0 '.format(
        question = question,
        options = {
            True:   '[Y/n]',
            False:  '[y/N]'
        }.get(default, '[y/n]')
    ))
    while True:
        choice = raw_input(prompt).lower()
        if choice in set(('yes', 'y')):
            return True
        elif choice in set(('no', 'n')):
            return False
        elif default is not None and not choice.strip():
            return default
        else:
            print("Invalid response")


def ask(prompt, multiline=False):
    if not multiline:
        return raw_input(cstr('^1{}:^0 ').format(prompt))

    ret = []
    try:
        ret.append(raw_input(cstr('^1{} (^^D to finish):^0 ').format(prompt)))
        while True:
            ret.append(raw_input())
    except EOFError:
        print("")

    return '\n'.join(ret)


def autouser(gitlab, prompt):
    user = None
    users = gitlab.list_users()

    readline.set_completer(UserCompleter(users).complete)
    while user is None:
        username = ask(prompt)
        user = gitlab.find_user(username)
        if username == '':
            break
        elif username == 'list':
            for u in users:
                print(cstr('^33@{}^0 ({})'.format(
                    u.username, u.name.encode('utf-8')
                )))
        elif user is None:
            print(cstr("^31User not found:^0 {}".format(username)))

    readline.set_completer(None)
    return user


def print_mr(app, mr):
    author = '<current-user>'
    title = ''
    assignee = "<no-one>"
    description = ''
    url = ''

    if 'title' in mr:
        title = mr['title']
    if 'assignee' in mr:
        assignee = "\033[33m@{}\033[0m ({})".format(
            mr['assignee'].username, mr['assignee'].name
        )
    if 'author' in mr:
        author = "\033[33m@{}\033[0m ({})".format(
            mr['author'].username, mr['author'].name
        )
    if 'description' in mr:
        description = mr['description']
    elif os.path.exists(MR_TEMPLATE_PATH):
        description = Template.load_from_file(MR_TEMPLATE_PATH).render()

    if 'remote_cache' in mr:
        url = urljoin(app.conf.gitlab_url, '/{}/merge_requests/{}'.format(
            app.conf.project,
            mr.remote_cache.iid
        ))

    print('')
    print(cstr("^1{}^0").format('-' * 80))
    print(cstr("^1{0:-^80}^0").format('{ Merge request preview }'))
    print(cstr("^1{}^0").format('-' * 80))
    print(cstr("^1URL:^0         ^34{}^0").format(url))
    print(cstr("^1Title:^0       ^32{}^0").format(title))
    print(cstr("^1Branch:^0      {}^0").format(git.current_branch()))
    print(cstr("^1Author:^0      {}").format(author))
    print(cstr("^1Assigned To:^0 {}").format(assignee))
    print(cstr("^1{0:-^80}^0").format('{ Description }'))
    print(description)
    print(cstr("\033[1m{}^0").format('-' * 80))
    print('')


def form_print(name, value):
    print(cstr('^1{}:^0 {}').format(name, value))




# -*- coding: utf-8 -*-
from peltak.config import MR_TEMPLATE_PATH, MR_CACHE_PATH
from .cache import CachedObj


def extract_mr(mrcache):
    mr = {
        'source_branch':    mrcache['source_branch'],
        'target_branch':    'master',
        'title':            mrcache['title'],
    }
    if 'description' in mrcache:
        mr['description'] = mrcache['description']

    if 'assignee' in mrcache:
        mr['assignee_id'] = mrcache['assignee'].id
    return mr


def _field_changed(name, current, remote):
    if name in current:
        if name not in remote:
            return True

        c, r = current[name], remote[name]
        if isinstance(c, dict) and isinstance(r, dict):
            return c.id != r.id
        return current[name] != remote[name]
    elif name in remote:
        return remote[name] is not None
    else:
        return True


def mrdiff(mrcache):
    """ Return the changes in the MR compared to the local cache. """
    changed = []
    for field in ('title', 'description', 'assignee'):
        if _field_changed(field, mrcache, mrcache.remote_cache):
            changed.append(field)

    ret = {}
    for field in changed:
        ret[field] = mrcache[field]

    return ret


def fetch_template(app):
    """ Fetch MR template from an gitlab issue.

    The issue should be labeled gitlab-cli and have a title equal to the
    value of ``config.mr_template_title``.
    """
    r = app.gitlab.api.get('issues?labels={}&state=opened'.format(
        app.conf.issue_label
    ))
    if r.status_code == 200:
        for issue in r.json():
            if issue['title'] == app.conf.mr_template_title:
                return issue['description'].replace('\r\n', '\n')
    return None


def resetmr_if_closed(app):
    """ Check if cached MR was closed. If so, reset the local MR cache.

    :return: ``True`` if MR was reset, ``False`` otherwise
    """
    mrcache = CachedObj.load_file(MR_CACHE_PATH)
    if 'remote_cache' in mrcache:
        mr = app.gitlab.get_mr(app.conf.project_id, mrcache.remote_cache.iid)
        if mr.state != "opened":
            mrcache.reset()
            return True
    return False




# -*- coding: utf-8 -*-
from os.path import abspath, dirname, join, normpath


#----------------------------------------------------------------------------//
def projpath(path):
    return normpath(join(dirname(abspath(__file__)), '..', path))


MR_TEMPLATE_PATH = '.peltak/mr-template.rst'
MR_CACHE_PATH = ".peltak/mr-cache.json"
CONFIG_PATH = '.peltak/config.json'
LOG_FILENAME = '.peltak/gitlab-cli.log'
ACTIONS_PATH = 'peltak/actions'
ACTIONS_DIRS = (
    projpath('peltak/actions'),
    '.peltak/actions',
)




# -*- coding: utf-8 -*-
import re
import textwrap
from peltak.actions import actions
from peltak.core import cstr


@actions.register(
    'list-actions',
    desc="Prints all available actions.",
    cmdline=[{
        'args':     ('--porcelain',),
        'action':   'store_true',
        'dest':     'porcelain',
        'help': ("Show output in a format friendly for scripts "
                 "instead of humans")
    }]
)
def list_actions(app):
    if app.cl.porcelain:
        print(' '.join(p.name for p in actions.iter()))
    else:
        maxlen  = max(len(p.name) for p in actions.iter())
        fmt     = "  ^32{{name:{}}}^0 -- {{desc}}".format(maxlen)
        indent  = '{}'.format(' ' * (maxlen + 6))
        plugins = sorted(actions.iter(), key=lambda x: x.name)
        print("\033[1mAvailable actions:\033[0m")
        for plugin in plugins:
            text = plugin.meta.get('desc', 'NO DESCRIPTION').strip()
            desc = cstr(fmt).format(
                name=plugin.name,
                desc=re.sub(r'\s+', ' ', text)
            )
            lines = textwrap.wrap(desc, 80, subsequent_indent=indent)
            print('\n'.join(lines))
            print('')




# -*- coding: utf-8 -*-
import os.path
from peltak.actions import actions
from peltak.config import MR_TEMPLATE_PATH, MR_CACHE_PATH
from peltak.common import prompt
from peltak.common.cache import CachedObj
from peltak.common.utils import resetmr_if_closed
from peltak.core.template import Template


@actions.register(
    'preview',
    desc="Prints the preview of the merge request in its current state."
)
def action_preview(app):
    resetmr_if_closed(app)
    mrcache = CachedObj.load_file(MR_CACHE_PATH)
    if os.path.exists(MR_TEMPLATE_PATH):
        template = Template.load_from_file(MR_TEMPLATE_PATH)
        template.fill(mrcache)
        mrcache.description = template.render()
    prompt.print_mr(app, mrcache)




# -*- coding: utf-8 -*-
import re
import os
import os.path


class CommaSeparatedList(object):
    def __init__(self, string):
        self.items = [v.strip() for v in string.split(',')]

    def __contains__(self, item):
        return item in self.items


def ensure_dirs(path):
    """ Make sure all ancestors of *path* exist, create them otherwise. """
    dirpath = os.path.dirname(path)

    if not os.path.exists(dirpath):
        os.makedirs(dirpath)

    elif not os.path.isdir(dirpath):
        raise IOError("ensure_dirs(): parent directory is not a directory")


def cstr(text, nocolors=False):
    """ Helper function for coloring strings for output in shell.

    :arg text:      Color formatted string to convert to shell colors.
    :arg nocolors:  If set to ``True`` all color codes will be removed from
                    *text* but not replaced by the shell codes. Defaults
                    to ``False``.

    The *text* should be formatted using ``^`` symbol followed by the shell
    code. So ``\e[33m`` becomes ``^33``.
    """
    def on_sub(m):
        if nocolors:
            return m.group(1)
        else:
            return m.group(1) + "\033[{}m".format(m.group('code'))

    pttrn = r'([^^]|^)(\^(?P<code>[1390]\d?))'
    while re.search(pttrn, text):
        text = re.sub(pttrn, on_sub, text)
    return text.replace("^^", "^")




# -*- coding: utf-8 -*-
from __future__ import absolute_import
import re
from six import PY2, PY3
from collections import OrderedDict
from ..core import git
from ..core.api import Gitlab
from ..common.cache import CachedObj
from ..config import CONFIG_PATH
from ..actions import actions
from . import forms
actions.discover()
if PY2:
    from urlparse import urlparse
elif PY3:
    from urllib.parse import urlparse


class AppContext(object):
    """

    **Members**
    :conf:      App config. Will save changes to disk on every update.
    :cl:        Parsed command line arguments.
    :gilab:     Gitlab API client.
    """
    def __init__(self, cmdline):
        self.conf = CachedObj.load_file(CONFIG_PATH)
        self.cl = cmdline
        self.gitlab = None
        self.load_config()

    def _parse_origin(self, origin):
        m = re.match(r'.*?@(?P<url>[^:]+):(?P<project>.*?).git', origin)
        if m:
            return 'https://{}'.format(m.group('url')), m.group('project')

        url = urlparse(origin)
        if url.path.endswith('.git'):
            self.conf.project = url.path[1:-4]
        else:
            self.conf.project = url.path[1:]

    def load_config(self):
        url, project = self._parse_origin(git.get_origin())
        defaults = OrderedDict({
            'project':           project,
            'gitlab_url':        '{url}'.format(url=url),
            'mr_template_title': 'mr-template',
            'issue_label':       'peltak',
            'verify_ssl':        True,
        })
        for name, value in defaults.items():
            if name not in self.conf:
                setattr(self.conf, name, value)

    def authenticate(self, get_user_details_fn=forms.login):
        """ This should authenticate the user against gitlab.

        :arg cmdline:   The return value from ArgumentParser.parse_args()
        :return:    Authenticated Gitlab instance ready for communication with
                    the API.

        It can be an interactive process where user provides his details.
        """
        token = self.conf.get('token')
        self.gitlab = Gitlab(
            url        = self.conf.gitlab_url,
            token      = token,
            verify_ssl = self.conf.verify_ssl
        )
        if not self.gitlab.logged_in:
            user, pw, save = get_user_details_fn()
            user = self.gitlab.login(user, pw)
            self.gitlab.token = user.private_token
            if save:
                self.conf.token = self.gitlab.token

        # Cache current project ID if missing
        if 'project_id' not in self.conf:
            project = self.gitlab.get_project(self.conf.project)
            self.conf.project_id = project.id




#!/usr/bin/env python2
# -*- coding: utf-8 -*-
import re
import os.path
from collections import OrderedDict


class Template(object):
    """ A simple implementation of a text template for merge requests.

    The class has 2 properties that are filled:

    :context:R
    """
    def __init__(self, content=None):
        self.content = content

    @classmethod
    def load_from_file(self, path):
        """ Load the template from file.

        During the load, the template will be preprocessed and ``context`` and
        ``checklist`` will be extracted.
        """
        if not os.path.exists(path):
            return None

        with open(path) as fp:
            content = fp.read()
            return Template(content)

    @property
    def content(self):
        """ Return the template content. """
        return self._content

    @content.setter
    def content(self, value):
        """ Set template content.

        This will process the template and extract placeholders and the
        checklist.
        """
        self._content = value
        self.context = {}
        if value is not None:
            self.context = self._extract_context(value)
            self.checklist = self._extract_checklist(value)

    def _extract_context(self, text):
        ret = OrderedDict()
        if text is not None:
            for m in re.finditer(r'\$\{(.*?)\}', text):
                name = m.group(1)
                if name not in ret:
                    ret[name] = ''
        return ret

    def _extract_checklist(self, text):
        ret = OrderedDict()
        if text is not None:
            for m in re.finditer(r'^\- \[ \] (.*)$', text, flags=re.M):
                question = m.group(1)
                if question not in ret:
                    ret[question] = None
        return ret

    def fill(self, src):
        """ Fill in the context and checklist from a given dict like object."""
        for key in self.context:
            if key in src:
                self.context[key] = src[key]

        for key in self.checklist:
            if key in src:
                self.checklist[key] = src[key]

    def render(self):
        """ Render the template to string.

        This will replace all the palceholders and the update the checklist
        according to values stored in ``self.checklist`` and ``self.context``.
        """
        out = self.content

        # Fill in the context
        for name, value in self.context.items():
            pttrn = r'\$\{{{}\}}'.format(re.escape(name))
            out = re.sub(pttrn, value or '<MISSING>', out)

        # Fill in the checklist
        for todo, complete in self.checklist.items():
            pttrn = r'^- \[ \] {}$'.format(re.escape(todo))
            item = '- [{}] {}'.format('x' if complete else ' ', todo)
            out = re.sub(pttrn, item, out, flags=re.M)

        return out




# -*- coding: utf-8 -*-
from peltak.actions import actions
from peltak.core import cstr


def str2bool(s):
    return s.lower() in ['true', 't', 'yes', '1', 'on']


@actions.register(
    'config',
    desc=cstr('\n'.join([
        "Manage configuration for the current project.",
        "",
        "EXAMPLES",
        "",
        "Disable SSL verification:",
        "  ^1$ %(prog)s config set-bool verify_ssl false^0",
        "",
        "Set the project id:",
        "  ^1$ %(prog)s config set-int project_id 10^0",
        "",
        "Set gitlab url:",
        "  ^1$ %(prog)s config set gitlab_url https://my.gitlab.url^0",
    ])),
    cmdline=[{
        'args':     ('action',),
        'nargs':    '?',
        'default':  'list',
        'help': ("Action to take: list/get/set/set-int/set-bool")
    }, {
        'args':     ('name',),
        'nargs':    '?',
        'help': ("If action is get or set, this should be the config "
                 "variable name.")
    }, {
        'args':     ('value',),
        'nargs':    '?',
        'help': ("If action is 'set', this should be the config value")
    }]
)
def manage_config(app):
    #action = app.cl.action.lower()
    action = {
        'list':     config_list,
        'get':      config_get,
        'set':      config_set,
        'set-int':  lambda a: config_set(a, vtype=int),
        'set-bool': lambda a: config_set(a, vtype=str2bool),
    }.get(
        app.cl.action.lower(),
        lambda a: None
    )
    action(app)


def config_set(app, vtype=str):
    name  = app.cl.name
    value = vtype(app.cl.value)
    setattr(app.conf, name, value)
    config_get(app)


def config_get(app):
    print(cstr("  ^32{name}^0 = {value}").format(
        name  = app.cl.name,
        value = app.conf[app.cl.name],
    ))


def config_list(app):
    maxlen = max(len(name) for name in app.conf.keys())
    fmt    = "  ^32{{name:{}}}^0 = {{value}}".format(maxlen)
    for name, value in app.conf.items():
        print(cstr(fmt).format(name=name, value=value))




import os
from setuptools import setup


def read(fname):
    return open(os.path.join(os.path.dirname(__file__), fname)).read()


setup(
    name = "peltak",
    version = "0.9.8",
    author = "Mateusz 'novo' Klos",
    author_email = "novopl@gmail.com",
    license = "MIT",
    keywords = "project management git gitlab",
    url = "http://github.com/novopl/peltak",
    entry_points={
        'console_scripts': [
            'peltak = peltak.runner:main'
        ]
    },
    packages=[
        'peltak',
        'peltak.actions',
        'peltak.common',
        'peltak.core',
        'peltak.system',
    ],
    description = ("Command line utility to improve your daily workflow."
                   "The main focus so far is making a gitlab merge request."),
    long_description=read('README.rst'),
    install_requires = [
        'six', 'libigor', 'requests'
    ],
    classifiers=[
        "Development Status :: 4 - Beta",
        "Topic :: Utilities",
        "Intended Audience :: Developers",
        "Programming Language :: Python",
        "License :: OSI Approved :: MIT License",
    ],
)



# -*- coding: utf-8 -*-
import sys
from time import sleep
from peltak.actions import actions
from peltak.core import git


@actions.register(
    'mr-cleanup',
    desc=("Cleanup after merged MR. It will remove the merged branch localy "
          "and remotely. If the user was on the deleted branch, it will "
          "checkout master first")
)
def mr_cleanup(app):
    sys.stdout.write('Pushing in 3..')
    sys.stdout.flush()
    sleep(1)
    sys.stdout.write('\rPushing in 2..')
    sys.stdout.flush()
    sleep(1)
    sys.stdout.write('\rPushing in 1..')
    sys.stdout.flush()
    sleep(1)
    sys.stdout.write('\r' + (' ' * 80) + '\r')
    sys.stdout.flush()
    sleep(1)
    git.push_current()

